<%@ WebHandler Language="C#" Class="stylesheet" %>

using System;
using System.IO;
using System.Web;
using System.Text;
using System.Text.RegularExpressions;

public class stylesheet : IHttpHandler
{

  public bool IsReusable
  {
    get { return false; }
  }

  public void ProcessRequest(HttpContext context)
  {
    if (!string.IsNullOrEmpty(context.Request.QueryString["stylesheets"]))
    {
      string[] relativeFiles = context.Request.QueryString["stylesheets"].Split(',');
      string[] absoluteFiles = new string[relativeFiles.Length];      

      for (int i = 0; i < relativeFiles.Length; i++)
      {
        string file = relativeFiles[i];
        if (file.EndsWith(".css"))
        {
          string absoluteFile = context.Server.MapPath(file);
          WriteContent(context, absoluteFile);
          absoluteFiles[i] = absoluteFile;
        }
      }
      
      SetHeaders(context, absoluteFiles);
    }
  }

  /// <summary>
  /// Writes the content of the individual stylesheets to the response stream.
  /// </summary>
  private void WriteContent(HttpContext context, string file)
  {
    using (StreamReader reader = new StreamReader(file))
    {
      string body = StripWhitespace(reader.ReadToEnd());
      context.Response.Write(body);
    }
  }

  /// <summary>
  /// Strips the whitespace from any .css file.
  /// </summary>
  private static string StripWhitespace(string body)
  {
    body = body.Replace("  ", String.Empty);
    body = body.Replace(Environment.NewLine, String.Empty);
    body = body.Replace("\t", string.Empty);
    body = body.Replace(" {", "{");
    body = body.Replace(" :", ":");
    body = body.Replace(": ", ":");
    body = body.Replace(", ", ",");
    body = body.Replace("; ", ";");
    body = body.Replace(";}", "}");    
    body = Regex.Replace(body, @"(?<=[>])\s{2,}(?=[<])|(?<=[>])\s{2,}(?=&nbsp;)|(?<=&ndsp;)\s{2,}(?=[<])", String.Empty);
    // This line can cause trouble on some stylesheets. It removes comments.
    body = Regex.Replace(body, @"/\*[^\*]*\*+([^/\*]*\*+)*/", "$1");

    return body;
  }

  /// <summary>
  /// This will make the browser and server keep the output
  /// in its cache and thereby improve performance.
  /// </summary>
  private static void SetHeaders(HttpContext context, string[] files)
  {
    context.Response.ContentType = "text/css";
    context.Response.AddFileDependencies(files);
    context.Response.Cache.VaryByParams["stylesheets"] = true;
    context.Response.Cache.SetETagFromFileDependencies();
    context.Response.Cache.SetLastModifiedFromFileDependencies();
    context.Response.Cache.SetValidUntilExpires(true);
    context.Response.Cache.SetExpires(DateTime.Now.AddDays(7));
    context.Response.Cache.SetCacheability(HttpCacheability.Public);
  }

}